from abaqus import *
from abaqusConstants import *            
from caeModules import *
from driverUtils import executeOnCaeStartup
from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *
from odbAccess import *

from numpy import *

uhel_vystrelu           =  17.900000000000016   #deg

vyska_luku              =  1.0e0   #m
predsazeni_u_rukojeti   =  1.0e-1  #m
rukojet_luku            =  0.4     #m
sirka_luku_u_rukojeti   =  4.0e-2  #m
prumer_sipu             =  5.0e-3  #m
delka_sipu              =  1.0e0   #m
tenka_cast_sipu         =  0.9e0
prumer_diry_pro_sip     =  6.0e-3  #m

sirka_profilu           =  1.0e-1  #m

zatizeni_sipu           =  0.3     #m
tuhost_tetivy           =  1.0e9   #N/m

prumer_tetivy           =  2e-3    #m

tloustka_jednosmer      =  5.0e-3
tloustka_tkanina        = 10.0e-3

material_jednosmer      = array([100.e9, 10.e9, 0.25, 5.e9, 5.e9, 5.e9])
hustota_jednosmer       = 1680  
material_tkanina        = array([40.e9, 40.e9, 0.25, 3.e9, 3.e9, 3.e9])
hustota_tkanina         = 2100  
material_ocel           = array([200e9, 0.3])
hustota_ocel            = 7680 
###REGULATIONS###

a_FEA                 = 2.5e-2
a_TOL                 = 1e-3*a_FEA

pocet_int_bodu        = 7

no_steps              = 10

built_model           = bool(1)
perform_analysis      = bool(1)
read_results          = bool(1)

timePeriod            =9.0 
maxNumInc             =1000000 
initialInc            =1e-5 
minInc                =1e-5
maxInc                =1. 

###BUILT MODEL###

if built_model:

    mdb = Mdb()

    mdb.models.changeKey(fromName='Model-1', toName='model')
    
    #profil
    s = mdb.models['model'].ConstrainedSketch(
        name='profil', 
        sheetSize=1.0)
    s.Arc3Points(
        point1=(0.0,  vyska_luku/2.0), 
        point2=(0.0, -vyska_luku/2.0), 
        point3=(predsazeni_u_rukojeti, 0.0))
    p = mdb.models['model'].Part(
        name='profil', 
        dimensionality=THREE_D, 
        type=DEFORMABLE_BODY)
    p.BaseShellExtrude(
        sketch=s, 
        depth=sirka_profilu)
        
    #orezavatko    
    s = mdb.models['model'].ConstrainedSketch(
        name='orezavatko', 
        sheetSize=1.0)
    s.rectangle(
        point1=(-2.0*vyska_luku, 2.0*vyska_luku), 
        point2=( 2.0*vyska_luku,-2.0*vyska_luku))
    s.EllipseByCenterPerimeter(
        center=(0.0, 0.0), 
        axisPoint1=(0.0, vyska_luku/2.0), 
        axisPoint2=(sirka_luku_u_rukojeti/2.0, 0.0))
    p = mdb.models['model'].Part(
        name='orezavatko', 
        dimensionality=THREE_D, 
        type=DEFORMABLE_BODY)
    p.BaseSolidExtrude(
        sketch=s, 
        depth=10.0*vyska_luku)
        
        
    #derovatko    
    s = mdb.models['model'].ConstrainedSketch(
        name='derovatko', 
        sheetSize=1.0)
    s.CircleByCenterPerimeter(
        center=(0.0, 0.0), 
        point1=(prumer_diry_pro_sip/2.0, 0.0))
    p = mdb.models['model'].Part(
        name='derovatko', 
        dimensionality=THREE_D, 
        type=DEFORMABLE_BODY)
    p.BaseSolidExtrude(
        sketch=s, 
        depth=10.0*vyska_luku)
        
    #sip    
    s = mdb.models['model'].ConstrainedSketch(
        name='sip', 
        sheetSize=1.0)
    s.Line(
        point1=(0.0, 0.0), 
        point2=(delka_sipu, 0.0))
    p = mdb.models['model'].Part(
        name='sip', 
        dimensionality=THREE_D, 
        type=DEFORMABLE_BODY)
    p.BaseWire(sketch=s)
        
    #nacpani do assembly
    a = mdb.models['model'].rootAssembly
    
    p = mdb.models['model'].parts['orezavatko']
    a.Instance(name='orezavatko', part=p, dependent=ON)
    p = mdb.models['model'].parts['profil']
    a.Instance(name='profil', part=p, dependent=ON)
    p = mdb.models['model'].parts['derovatko']
    a.Instance(name='derovatko', part=p, dependent=ON)              
    p = mdb.models['model'].parts['sip']
    a.Instance(name='sip', part=p, dependent=ON)
        
    #orezani    
    a = mdb.models['model'].rootAssembly

    a.rotate(
        instanceList=('orezavatko', ), 
        axisPoint=(0.0, 0.0, 0.0), 
        axisDirection=(0.0, 1.0, 0.0), 
        angle=90.0)
    a.translate(
        instanceList=('orezavatko', ), 
        vector=(-5.0*vyska_luku, 0.0, 0.0))         
        
    a.InstanceFromBooleanCut(
        name='luk_bez_diry', 
        instanceToBeCut=a.instances['profil'], 
        cuttingInstances=(a.instances['orezavatko'], ), 
        originalInstances=DELETE)

    a.rotate(
        instanceList=('derovatko', ), 
        axisPoint=(0.0, 0.0, 0.0), 
        axisDirection=(0.0, 1.0, 0.0), 
        angle=90.0)
        
    a.InstanceFromBooleanCut(
        name='luk', 
        instanceToBeCut=mdb.models['model'].rootAssembly.instances['luk_bez_diry-1'], 
        cuttingInstances=(a.instances['derovatko'], ), 
        originalInstances=DELETE)        
    
    del a.features['luk-1']
    
    #datum planes - luk
    p = mdb.models['model'].parts['luk']
    plane_zero      = p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=0.0)
    plane_symmetry  = p.DatumPlaneByPrincipalPlane(principalPlane=XYPLANE, offset=0.0)
    plane_top       = p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=rukojet_luku/2.0)
    plane_bottom    = p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=-rukojet_luku/2.0)
    p = mdb.models['model'].parts['sip']
    plane_sip    = p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=tenka_cast_sipu)

    #for some reason the only one half is made from cut oparation therefore the mirroring is necessary
    p = mdb.models['model'].parts['luk']
    d = p.datums
    p.Mirror(
      mirrorPlane=d[plane_symmetry.id], 
      keepOriginal=ON,           
      keepInternalBoundaries=ON) 

    #partitions of luk
    p = mdb.models['model'].parts['luk']
    f = p.faces
    
    pickedFaces = f.findAt((predsazeni_u_rukojeti,0.0,-sirka_luku_u_rukojeti/2.0),)
    p.PartitionFaceByDatumPlane(datumPlane=d[plane_top.id], faces=pickedFaces)
    pickedFaces = f.findAt((predsazeni_u_rukojeti,0.0,sirka_luku_u_rukojeti/2.0),)
    p.PartitionFaceByDatumPlane(datumPlane=d[plane_top.id], faces=pickedFaces)
    
    pickedFaces = f.findAt((predsazeni_u_rukojeti,0.0,-sirka_luku_u_rukojeti/2.0),)
    p.PartitionFaceByDatumPlane(datumPlane=d[plane_bottom.id], faces=pickedFaces)
    pickedFaces = f.findAt((predsazeni_u_rukojeti,0.0,sirka_luku_u_rukojeti/2.0),)
    p.PartitionFaceByDatumPlane(datumPlane=d[plane_bottom.id], faces=pickedFaces)
    
    pickedFaces = f.findAt((predsazeni_u_rukojeti,0.0,-sirka_luku_u_rukojeti/2.0),)
    p.PartitionFaceByDatumPlane(datumPlane=d[plane_zero.id], faces=pickedFaces)
    pickedFaces = f.findAt((predsazeni_u_rukojeti,0.0,sirka_luku_u_rukojeti/2.0),)
    p.PartitionFaceByDatumPlane(datumPlane=d[plane_zero.id], faces=pickedFaces)

    #partitions of sip
    p = mdb.models['model'].parts['sip']
    e = p.edges
    x_min  = -10.*vyska_luku 
    y_min  = -10.*vyska_luku 
    z_min  = -10.*vyska_luku
    x_max  =  10.*vyska_luku
    y_max  =  10.*vyska_luku
    z_max  =  10.*vyska_luku    
    pickedEdges = e.getByBoundingBox(x_min-a_TOL,y_min-a_TOL,z_min-a_TOL,x_max+a_TOL,y_max+a_TOL,z_max+a_TOL)
    p.PartitionEdgeByDatumPlane(datumPlane=d[plane_sip.id], edges=pickedEdges)        
   
    #sety faces - luk
    p = mdb.models['model'].parts['luk']
    f = p.faces
    
    x_min  = -10.*vyska_luku 
    y_min  = -rukojet_luku/2.0  
    z_min  = -10.*vyska_luku
    x_max  =  10.*vyska_luku
    y_max  = rukojet_luku/2.0
    z_max  =  10.*vyska_luku    
    faces = f.getByBoundingBox(x_min-a_TOL,y_min-a_TOL,z_min-a_TOL,x_max+a_TOL,y_max+a_TOL,z_max+a_TOL)    
    p.Set(faces=faces, name='rukojet')

    x_min  = -10.*vyska_luku 
    y_min  = rukojet_luku/2.0  
    z_min  = -10.*vyska_luku
    x_max  =  10.*vyska_luku
    y_max  =  10.*vyska_luku
    z_max  =  10.*vyska_luku 
    faces  = f.getByBoundingBox(x_min-a_TOL,y_min-a_TOL,z_min-a_TOL,x_max+a_TOL,y_max+a_TOL,z_max+a_TOL)
    p.Set(faces=faces, name='zbytek_nahore')
    
    x_min  = -10.*vyska_luku 
    y_min  = -10.*vyska_luku  
    z_min  = -10.*vyska_luku
    x_max  =  10.*vyska_luku
    y_max  = -rukojet_luku/2.0
    z_max  =  10.*vyska_luku 
    faces  = f.getByBoundingBox(x_min-a_TOL,y_min-a_TOL,z_min-a_TOL,x_max+a_TOL,y_max+a_TOL,z_max+a_TOL)        
    p.Set(faces=faces, name='zbytek_dole')
    
    #materials
    mdb.models['model'].Material(name='jednosmer')
    mdb.models['model'].materials['jednosmer'].Elastic(
        type=LAMINA, 
        table=(material_jednosmer, ))
    mdb.models['model'].materials['jednosmer'].Density(table=((hustota_jednosmer, ), ))        
        
    mdb.models['model'].Material(name='tkanina')
    mdb.models['model'].materials['tkanina'].Elastic(
        type=LAMINA, 
        table=(material_tkanina, ))
    mdb.models['model'].materials['tkanina'].Density(table=((hustota_tkanina, ), ))         
        
    mdb.models['model'].Material(name='ocel')
    mdb.models['model'].materials['ocel'].Elastic(
        type=ISOTROPIC, 
        table=(material_ocel, ))
    mdb.models['model'].materials['ocel'].Density(table=((hustota_ocel, ), ))        
    
    #composite layup - nahore
    p = mdb.models['model'].parts['luk']

    region1=p.sets['zbytek_nahore']

    s = p.faces
    x_min  = -10.*vyska_luku 
    y_min  = rukojet_luku/2.0  
    z_min  = -10.*vyska_luku
    x_max  =  10.*vyska_luku
    y_max  =  10.*vyska_luku
    z_max  =  10.*vyska_luku 
    side1Faces  = s.getByBoundingBox(x_min-a_TOL,y_min-a_TOL,z_min-a_TOL,x_max+a_TOL,y_max+a_TOL,z_max+a_TOL)
    normalAxisRegion = p.Surface(side1Faces=side1Faces, name='normal_axis_zbytek_nahore')
    
    e = p.edges
    for ee in e:
      pointon=array(ee.pointOn)
      if pointon[0][2]==0.0:
        if pointon[0][1]>rukojet_luku/2.0:
          edges=ee
    primaryAxisRegion = (edges,)
    
    compositeLayup = mdb.models['model'].parts['luk'].CompositeLayup(
        name='CompositeLayup_nahore', 
        description='', 
        elementType=SHELL, 
        offsetType=BOTTOM_SURFACE, 
        symmetric=False, 
        thicknessAssignment=FROM_SECTION)
    compositeLayup.Section(
        preIntegrate=OFF, 
        integrationRule=SIMPSON, 
        thicknessType=UNIFORM, 
        poissonDefinition=DEFAULT, 
        temperature=GRADIENT, 
        useDensity=OFF)
    compositeLayup.CompositePly(
        suppressed=False, 
        plyName='vrtsva_jednosmer', 
        region=region1, 
        material='jednosmer', 
        thicknessType=SPECIFY_THICKNESS, 
        thickness=tloustka_jednosmer, 
        orientationType=SPECIFY_ORIENT, 
        orientationValue=0.0, 
        additionalRotationType=ROTATION_NONE, 
        additionalRotationField='', 
        axis=AXIS_3, 
        angle=0.0, 
        numIntPoints=pocet_int_bodu)
    compositeLayup.ReferenceOrientation(
        orientationType=DISCRETE, 
        localCsys=None, 
        additionalRotationType=ROTATION_NONE, 
        angle=0.0, 
        additionalRotationField='', 
        axis=AXIS_3, 
        stackDirection=STACK_3, 
        normalAxisDefinition=SURFACE, 
        normalAxisRegion=normalAxisRegion, 
        normalAxisDirection=AXIS_3, 
        flipNormalDirection=False, 
        primaryAxisDefinition=EDGE, 
        primaryAxisRegion=primaryAxisRegion, 
        primaryAxisDirection=AXIS_1, 
        flipPrimaryDirection=False)
        
    #composite layup - dole
    p = mdb.models['model'].parts['luk']

    region1=p.sets['zbytek_dole']

    s = p.faces
    x_min  = -10.*vyska_luku 
    y_min  = -10.*vyska_luku  
    z_min  = -10.*vyska_luku
    x_max  =  10.*vyska_luku
    y_max  = -rukojet_luku/2.0
    z_max  =  10.*vyska_luku 
    side1Faces  = s.getByBoundingBox(x_min-a_TOL,y_min-a_TOL,z_min-a_TOL,x_max+a_TOL,y_max+a_TOL,z_max+a_TOL)
    normalAxisRegion = p.Surface(side1Faces=side1Faces, name='normal_axis_zbytek_dole')
    
    e = p.edges
    for ee in e:
      pointon=array(ee.pointOn)
      if pointon[0][2]==0.0:
        if pointon[0][1]<-rukojet_luku/2.0:
          edges=ee
    primaryAxisRegion = (edges,)
    
    compositeLayup = mdb.models['model'].parts['luk'].CompositeLayup(
        name='CompositeLayup_dole', 
        description='', 
        elementType=SHELL, 
        offsetType=BOTTOM_SURFACE, 
        symmetric=False, 
        thicknessAssignment=FROM_SECTION)
    compositeLayup.Section(
        preIntegrate=OFF, 
        integrationRule=SIMPSON, 
        thicknessType=UNIFORM, 
        poissonDefinition=DEFAULT, 
        temperature=GRADIENT, 
        useDensity=OFF)
    compositeLayup.CompositePly(
        suppressed=False, 
        plyName='vrtsva_jednosmer', 
        region=region1, 
        material='jednosmer', 
        thicknessType=SPECIFY_THICKNESS, 
        thickness=tloustka_jednosmer, 
        orientationType=SPECIFY_ORIENT, 
        orientationValue=0.0, 
        additionalRotationType=ROTATION_NONE, 
        additionalRotationField='', 
        axis=AXIS_3, 
        angle=0.0, 
        numIntPoints=pocet_int_bodu)
    compositeLayup.ReferenceOrientation(
        orientationType=DISCRETE, 
        localCsys=None, 
        additionalRotationType=ROTATION_NONE, 
        angle=0.0, 
        additionalRotationField='', 
        axis=AXIS_3, 
        stackDirection=STACK_3, 
        normalAxisDefinition=SURFACE, 
        normalAxisRegion=normalAxisRegion, 
        normalAxisDirection=AXIS_3, 
        flipNormalDirection=False, 
        primaryAxisDefinition=EDGE, 
        primaryAxisRegion=primaryAxisRegion, 
        primaryAxisDirection=AXIS_1, 
        flipPrimaryDirection=False)             
    
    #composite layup - rukojet
    p = mdb.models['model'].parts['luk']

    region1=p.sets['rukojet']

    s = p.faces
    x_min  = -10.*vyska_luku 
    y_min  = -10.*vyska_luku  
    z_min  = -10.*vyska_luku
    x_max  =  10.*vyska_luku
    y_max  = -rukojet_luku/2.0
    z_max  =  10.*vyska_luku 
    side1Faces  = s.getByBoundingBox(x_min-a_TOL,y_min-a_TOL,z_min-a_TOL,x_max+a_TOL,y_max+a_TOL,z_max+a_TOL)
    normalAxisRegion = p.Surface(side1Faces=side1Faces, name='normal_axis_rukojet')
    
    e = p.edges
    for ee in e:
      pointon=array(ee.pointOn)
      if pointon[0][2]==0.0:
        if pointon[0][1]>-rukojet_luku/2.0 and pointon[0][1]<rukojet_luku/2.0:
          edges=ee
    primaryAxisRegion = (edges,)
    
    compositeLayup = mdb.models['model'].parts['luk'].CompositeLayup(
        name='CompositeLayup_rukojet', 
        description='', 
        elementType=SHELL, 
        offsetType=BOTTOM_SURFACE, 
        symmetric=False, 
        thicknessAssignment=FROM_SECTION)
    compositeLayup.Section(
        preIntegrate=OFF, 
        integrationRule=SIMPSON, 
        thicknessType=UNIFORM, 
        poissonDefinition=DEFAULT, 
        temperature=GRADIENT, 
        useDensity=OFF)
    compositeLayup.CompositePly(
        suppressed=False, 
        plyName='vrtsva_jednosmer', 
        region=region1, 
        material='jednosmer', 
        thicknessType=SPECIFY_THICKNESS, 
        thickness=tloustka_jednosmer, 
        orientationType=SPECIFY_ORIENT, 
        orientationValue=0.0, 
        additionalRotationType=ROTATION_NONE, 
        additionalRotationField='', 
        axis=AXIS_3, 
        angle=0.0, 
        numIntPoints=pocet_int_bodu)
    compositeLayup.CompositePly(
        suppressed=False, 
        plyName='vrtsva_tkanina', 
        region=region1, 
        material='tkanina', 
        thicknessType=SPECIFY_THICKNESS, 
        thickness=tloustka_tkanina, 
        orientationType=SPECIFY_ORIENT, 
        orientationValue=45.0, 
        additionalRotationType=ROTATION_NONE, 
        additionalRotationField='', 
        axis=AXIS_3, 
        angle=0.0, 
        numIntPoints=pocet_int_bodu)
    compositeLayup.ReferenceOrientation(
        orientationType=DISCRETE, 
        localCsys=None, 
        additionalRotationType=ROTATION_NONE, 
        angle=0.0, 
        additionalRotationField='', 
        axis=AXIS_3, 
        stackDirection=STACK_3, 
        normalAxisDefinition=SURFACE, 
        normalAxisRegion=normalAxisRegion, 
        normalAxisDirection=AXIS_3, 
        flipNormalDirection=False, 
        primaryAxisDefinition=EDGE, 
        primaryAxisRegion=primaryAxisRegion, 
        primaryAxisDirection=AXIS_1, 
        flipPrimaryDirection=False)    
    
    #section and profile of sip
    mdb.models['model'].CircularProfile(
        name='profil_sip', 
        r=prumer_sipu/2.0)
    mdb.models['model'].CircularProfile(
        name='profil_sip_predek', 
        r=prumer_sipu) 
    mdb.models['model'].BeamSection(
        name='section_sip', 
        integration=DURING_ANALYSIS, 
        poissonRatio=0.0, 
        profile='profil_sip', 
        material='ocel', 
        temperatureVar=LINEAR, 
        consistentMassMatrix=False)
    mdb.models['model'].BeamSection(
        name='section_sip_predek', 
        integration=DURING_ANALYSIS, 
        poissonRatio=0.0, 
        profile='profil_sip_predek', 
        material='ocel', 
        temperatureVar=LINEAR, 
        consistentMassMatrix=False)        
    p = mdb.models['model'].parts['sip']
    e = p.edges
    pickedEdges = e.findAt((0.0,0.0,0.0),)
    region = p.Set(edges=e[pickedEdges.index:pickedEdges.index+1], name='sip')
    p.SectionAssignment(
        region=region, 
        sectionName='section_sip', 
        offset=0.0, 
        offsetType=MIDDLE_SURFACE, 
        offsetField='', 
        thicknessAssignment=FROM_SECTION) 
    p.assignBeamSectionOrientation(
        region=region, 
        method=N1_COSINES, 
        n1=(0.0, 1.0, 0.0))
    pickedEdges = e.findAt((delka_sipu,0.0,0.0),)
    region = p.Set(edges=e[pickedEdges.index:pickedEdges.index+1], name='sip_predek')
    p.SectionAssignment(
        region=region, 
        sectionName='section_sip_predek', 
        offset=0.0, 
        offsetType=MIDDLE_SURFACE, 
        offsetField='', 
        thicknessAssignment=FROM_SECTION) 
    p.assignBeamSectionOrientation(
        region=region, 
        method=N1_COSINES, 
        n1=(0.0, 1.0, 0.0))        

    #mesh - luk    
    p = mdb.models['model'].parts['luk']
    f = p.faces
    elemType1 = mesh.ElemType(elemCode=S4, elemLibrary=STANDARD, 
        secondOrderAccuracy=OFF)
    elemType2 = mesh.ElemType(elemCode=S3, elemLibrary=STANDARD)
    x_min  = -10.*vyska_luku 
    y_min  = -10.*vyska_luku  
    z_min  = -10.*vyska_luku
    x_max  =  10.*vyska_luku
    y_max  =  10.*vyska_luku
    z_max  =  10.*vyska_luku 
    faces  = f.getByBoundingBox(x_min-a_TOL,y_min-a_TOL,z_min-a_TOL,x_max+a_TOL,y_max+a_TOL,z_max+a_TOL)  
    p.setElementType(regions=(faces, ), elemTypes=(elemType1, elemType2))
    p.setMeshControls(regions=faces, elemShape=QUAD, algorithm=MEDIAL_AXIS)
    p.seedPart(size=a_FEA, deviationFactor=0.1, minSizeFactor=0.1)
    p.generateMesh()
    
    #mesh - sip
    p = mdb.models['model'].parts['sip']
    elemType1 = mesh.ElemType(elemCode=B31, elemLibrary=EXPLICIT)
#     elemType1 = mesh.ElemType(elemCode=T3D2, elemLibrary=EXPLICIT)
    e = p.edges
    x_min  = -10.*vyska_luku 
    y_min  = -10.*vyska_luku  
    z_min  = -10.*vyska_luku
    x_max  =  10.*vyska_luku
    y_max  =  10.*vyska_luku
    z_max  =  10.*vyska_luku 
    edges  =  e.getByBoundingBox(x_min-a_TOL,y_min-a_TOL,z_min-a_TOL,x_max+a_TOL,y_max+a_TOL,z_max+a_TOL)
    pickedRegions =(edges, )
    p.setElementType(
        regions=pickedRegions, 
        elemTypes=(elemType1, ))
    p.seedPart(size=a_FEA, deviationFactor=0.1, minSizeFactor=0.1)
    p.generateMesh()    
    
    #pridani luku do assembly
    p = mdb.models['model'].parts['luk']
    a.Instance(name='luk', part=p, dependent=ON) 

    #load step
    mdb.models['model'].StaticStep(
        name='load', 
        previous='Initial',
        timeIncrementationMethod=FIXED)
    mdb.models['model'].ImplicitDynamicsStep(
        name='release', 
        previous='load',
        timePeriod=timePeriod,             
        maxNumInc=maxNumInc,              
        initialInc=initialInc,            
        minInc= minInc,                
        maxInc=maxInc,                
        nohaf=OFF)        

    #sets for connections of tetiva and luk
    
    a = mdb.models['model'].rootAssembly
    v = a.instances['luk'].vertices
    verts = v.findAt((0.0,vyska_luku/2.0,0.0),)
    uchyceni_luk_nahore=a.Set(vertices=v[verts.index:verts.index+1], name='uchyceni_luk_nahore')
    
    a = mdb.models['model'].rootAssembly
    v = a.instances['luk'].vertices
    verts = v.findAt((0.0,-vyska_luku/2.0,0.0),)
    uchyceni_luk_dole=a.Set(vertices=v[verts.index:verts.index+1], name='uchyceni_luk_dole')
    
    a = mdb.models['model'].rootAssembly
    v = a.instances['sip'].vertices
    verts = v.findAt((0.0, 0.0,0.0),)
    uchyceni_sip_nazacatku=a.Set(vertices=v[verts.index:verts.index+1], name='uchyceni_sip_nazacatku')      

    a = mdb.models['model'].rootAssembly
    v = a.instances['sip'].vertices
    verts = v.findAt((delka_sipu, 0.0,0.0),)
    uchyceni_sip_nakonci=a.Set(vertices=v[verts.index:verts.index+1], name='uchyceni_sip_nakonci')

    #system souradnic
    a = mdb.models['model'].rootAssembly
    v0=a.sets['uchyceni_sip_nazacatku']
    v1=a.sets['uchyceni_sip_nakonci']
    v2=a.sets['uchyceni_luk_nahore']
    datum_natoceni = a.DatumCsysByThreePoints(
        point1=v1.vertices[0], 
        point2=v2.vertices[0], 
        name='smer_vystrelu', 
        coordSysType=CARTESIAN, 
        origin=v0.vertices[0])
        
    #BC
    e = a.instances['luk'].edges
    edges=()
    for edge in e:
        pointArray = array(edge.pointOn)[0][1:3]
        if sqrt(sum(pointArray**2.0))<=prumer_diry_pro_sip/2.0+a_TOL:
            edges = edges + (edge,) 
    mdb.models['model'].DisplacementBC(
        name='uchyceni_luk', 
        createStepName='Initial', 
        region=edges, 
        u1=SET, 
        u2=SET, 
        u3=SET, 
        ur1=SET, 
        ur2=SET, 
        ur3=SET, 
        amplitude=UNSET, 
        distributionType=UNIFORM, 
        fieldName='', 
        localCsys=mdb.models['model'].rootAssembly.datums[datum_natoceni.id])
    
    #load BC   
    a = mdb.models['model'].rootAssembly
    mdb.models['model'].TabularAmplitude(
        name='amplitude', 
        timeSpan=STEP, 
        smooth=SOLVER_DEFAULT, 
        data=((0.0, 0.0), (1.0, 1.0)))     
    mdb.models['model'].DisplacementBC(
        name='load', 
        createStepName='load', 
        region=uchyceni_sip_nazacatku, 
        u1=-zatizeni_sipu, 
        u2=0.0, 
        u3=0.0, 
        ur1=0.0, 
        ur2=0.0, 
        ur3=0.0, 
        amplitude='amplitude', 
        fixed=OFF, 
        distributionType=UNIFORM, 
        fieldName='', 
        localCsys=mdb.models['model'].rootAssembly.datums[datum_natoceni.id])
    mdb.models['model'].boundaryConditions['load'].deactivate('release')
    
    #gravity load
    mdb.models['model'].Gravity(
        name='gravity', 
        createStepName='release', 
        comp2=-9.81, 
        distributionType=UNIFORM, 
        field='') 
        
    #springs
    a = mdb.models['model'].rootAssembly
    rgn1pair0=a.sets['uchyceni_sip_nazacatku']
    rgn2pair0=a.sets['uchyceni_luk_dole']
    rgn1pair1=a.sets['uchyceni_sip_nazacatku']
    rgn2pair1=a.sets['uchyceni_luk_nahore']    
    region=((rgn2pair0, rgn1pair0),(rgn2pair1, rgn1pair1), )
    mdb.models['model'].rootAssembly.engineeringFeatures.TwoPointSpringDashpot(
        name='tetiva', 
        regionPairs=region, 
        axis=FIXED_DOF, 
        dof1=1, 
        dof2=1, 
        springBehavior=ON, 
        springStiffness=tuhost_tetivy, 
        dashpotBehavior=OFF, 
        dashpotCoefficient=0.0,
        orientation=mdb.models['model'].rootAssembly.datums[datum_natoceni.id])

    #rotace
    a = mdb.models['model'].rootAssembly
    a.rotate(
        instanceList=('sip', 'luk'), 
        axisPoint=(0.0, 0.0, 0.0), 
        axisDirection=(0.0, 0.0, 1.0), 
        angle=uhel_vystrelu) 
  
    #field and history outputs
    mdb.models['model'].fieldOutputRequests.changeKey(
        fromName='F-Output-1', 
        toName='normal')    
    mdb.models['model'].fieldOutputRequests['normal'].setValues(frequency=1)
    mdb.models['model'].fieldOutputRequests['normal'].setValues(
        variables=('S', 'U', 'RF', 'CF', 'CSTRESS'))    

    mdb.models['model'].FieldOutputRequest(
        name='normal_release', 
        createStepName='release', 
        variables=('S', 'U', 'V', 'A', 'RF', 'CF', 'CSTRESS'))
    mdb.models['model'].fieldOutputRequests['normal_release'].setValues(frequency=1)
    
    mdb.models['model'].FieldOutputRequest(
        name='kompozity', 
        createStepName='load', 
        variables=('S', 'TSHR', 'STH'), 
        layupNames=('luk.CompositeLayup_dole','luk.CompositeLayup_nahore', 'luk.CompositeLayup_rukojet' ), 
        layupLocationMethod=ALL_LOCATIONS, 
        rebar=EXCLUDE)
    mdb.models['model'].fieldOutputRequests['kompozity'].setValues(frequency=1)        

    mdb.models['model'].historyOutputRequests.changeKey(
        fromName='H-Output-1', 
        toName='normal')        
    mdb.models['model'].historyOutputRequests['normal'].setValues(frequency=1)        
    
    mdb.models['model'].keywordBlock.synchVersions(storeNodesAndElements=False)
    mdb.models['model'].keywordBlock.replace(72, """
*Spring, orientation=smer_vystrelu, elset=tetiva-spring, nonlinear, dependencies=1
1, 1                                                                              
0,0                                                                               
1e+09,1                                                                           
""") 
        
    #job
    mdb.Job(
        name='job', 
        model='model', 
        description='', 
        type=ANALYSIS, 
        atTime=None, 
        waitMinutes=0, 
        waitHours=0, 
        queue=None, 
        memory=90, 
        memoryUnits=PERCENTAGE, 
        getMemoryFromAnalysis=True, 
        explicitPrecision=DOUBLE_PLUS_PACK, 
        nodalOutputPrecision=FULL, 
        echoPrint=OFF, 
        modelPrint=OFF, 
        contactPrint=OFF, 
        historyPrint=OFF, 
        userSubroutine='', 
        scratch=os.getcwd(), 
        multiprocessingMode=DEFAULT, 
        numCpus=1, 
        numGPUs=0)

    mdb.saveAs(pathName='luk')
    
if perform_analysis:    

    mdb=openMdb(pathName='luk')
    
    mdb.jobs['job'].submit(
        consistencyChecking=ON, 
        datacheckJob = False, 
        continueJob = False)
    mdb.jobs['job'].waitForCompletion() 

if read_results:
    
    session.openOdb('job.odb')
    odb = session.odbs['job.odb']
    session.viewports['Viewport: 1'].setValues(displayedObject=odb)
    session.xyDataListFromField(
        odb=odb, 
        outputPosition=NODAL, 
        variable=(('U',  NODAL, ((COMPONENT, 'U1'), (COMPONENT, 'U2'), )), ), 
        nodeSets=('UCHYCENI_SIP_NAKONCI', )) 
    xy1 = session.xyDataObjects['U:U1 PI: SIP N: 3']
    xy2 = session.xyDataObjects['U:U2 PI: SIP N: 3']
    xy3 = combine(xy1, xy2)
    xyp = session.XYPlot('XYPlot-1')
    chartName = xyp.charts.keys()[0]
    chart = xyp.charts[chartName]
    c1 = session.Curve(xyData=xy3)
#     chart.setValues(curvesToPlot=(c1, ), ) 
    session.writeXYReport(fileName='vysledek.output', xyData=(c1, )) 
    session.odbs['job.odb'].close()                                                                     
